/*-
 * Copyright 2021 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.ade.foodwaterenergycitygml4j.landuse.model;


import org.citygml4j.builder.copy.CopyBuilder;
import org.citygml4j.model.citygml.ade.binding.ADEModelObject;
import org.citygml4j.model.common.visitor.FeatureFunctor;
import org.citygml4j.model.common.visitor.FeatureVisitor;
import org.citygml4j.model.common.visitor.GMLFunctor;
import org.citygml4j.model.common.visitor.GMLVisitor;
import org.citygml4j.model.gml.basicTypes.Code;
import org.citygml4j.model.gml.feature.AbstractFeature;
import org.citygml4j.model.gml.measures.Scale;



public class EnergySurvey extends AbstractFeature implements ADEModelObject{
	
	private static final long serialVersionUID = -7095068766106216686L;
	private Scale spaceHeatingDemand;
	private Scale spaceCoolingDemand;
	private Scale residentialElectricityDemand;
	private Scale electricityPotentialFromPVBuildings;
	private Scale biomassPrimaryEnergyPotential;
	private Scale biomassToEnergyWoods;
	private Scale biomassToBiogas;
	private Scale biomassToPlantOil;
	private Scale biomassToBioethanol;
	private Scale biomassToResidualSolidFuel;
	private Scale biomassToSolidFuel;
	private Scale biomassToElectricityPotential;
	private Scale biomassToThermalPotential;
	private Code vegetationCoverCategory;
	
	
	public Scale getSpaceHeatingDemand() {
		return spaceHeatingDemand;
	}
	
	
	public boolean isSpaceHeatingDemand() {
		return spaceHeatingDemand != null;
	}

	public void setSpaceHeatingDemand(Scale spaceHeatingDemand) {
		this.spaceHeatingDemand = spaceHeatingDemand;
	}

	public Scale getSpaceCoolingDemand() {
		return spaceCoolingDemand;
	}
	

	public boolean isSpaceCoolingDemand() {
		return spaceCoolingDemand!=null;
	}

	public void setSpaceCoolingDemand(Scale spaceCoolingDemand) {
		this.spaceCoolingDemand = spaceCoolingDemand;
	}

	public Scale getResidentialElectricityDemand() {
		return residentialElectricityDemand;
	}
	
	public boolean isResidentialElectricityDemand() {
		return residentialElectricityDemand != null;
	}

	public void setResidentialElectricityDemand(Scale residentialElectricityDemand) {
		this.residentialElectricityDemand = residentialElectricityDemand;
	}

	public Scale getElectricityPotentialFromPVBuildings() {
		return electricityPotentialFromPVBuildings;
	}
	

	public boolean isElectricityPotentialFromPVBuildings() {
		return electricityPotentialFromPVBuildings!=null;
	}

	public void setElectricityPotentialFromPVBuildings(Scale electricityPotentialFromPVBuildings) {
		this.electricityPotentialFromPVBuildings = electricityPotentialFromPVBuildings;
	}

	public Code getVegetationCoverCategory() {
		return vegetationCoverCategory;
	}
	

	public boolean isVegetationCoverCategory() {
		return vegetationCoverCategory != null;
	}

	public void setVegetationCoverCategory(Code vegetationCoverCategory) {
		this.vegetationCoverCategory = vegetationCoverCategory;
	}

	public Scale getBiomassPrimaryEnergyPotential() {
		return biomassPrimaryEnergyPotential;
	}
	
	public boolean isBiomassPrimaryEnergyPotential() {
		return biomassPrimaryEnergyPotential != null;
	}

	public void setBiomassPrimaryEnergyPotential(Scale biomassPrimaryEnergyPotential) {
		this.biomassPrimaryEnergyPotential = biomassPrimaryEnergyPotential;
	}

	public Scale getBiomassToEnergyWoods() {
		return biomassToEnergyWoods;
	}
	

	public boolean isBiomassToEnergyWoods() {
		return biomassToEnergyWoods !=null;
	}

	public void setBiomassToEnergyWoods(Scale biomassToEnergyWoods) {
		this.biomassToEnergyWoods = biomassToEnergyWoods;
	}

	public Scale getBiomassToBiogas() {
		return biomassToBiogas;
	}
	

	public boolean isBiomassToBiogas() {
		return biomassToBiogas != null;
	}

	public void setBiomassToBiogas(Scale biomassToBiogas) {
		this.biomassToBiogas = biomassToBiogas;
	}

	public Scale getBiomassToPlantOil() {
		return biomassToPlantOil;
	}
	

	public boolean isBiomassToPlantOil() {
		return biomassToPlantOil != null;
	}


	public void setBiomassToPlantOil(Scale biomassToPlantOil) {
		this.biomassToPlantOil = biomassToPlantOil;
	}

	public Scale getBiomassToBioethanol() {
		return biomassToBioethanol;
	}
	

	public boolean isBiomassToBioethanol() {
		return biomassToBioethanol != null;
	}

	public void setBiomassToBioethanol(Scale biomassToBioethanol) {
		this.biomassToBioethanol = biomassToBioethanol;
	}

	public Scale getBiomassToResidualSolidFuel() {
		return biomassToResidualSolidFuel;
	}
	

	public boolean isBiomassToResidualSolidFuel() {
		return biomassToResidualSolidFuel!=null;
	}

	public void setBiomassToResidualSolidFuel(Scale biomassToResidualSolidFuel) {
		this.biomassToResidualSolidFuel = biomassToResidualSolidFuel;
	}

	public Scale getBiomassToSolidFuel() {
		return biomassToSolidFuel;
	}
	

	public boolean isBiomassToSolidFuel() {
		return biomassToSolidFuel !=null;
	}

	public void setBiomassToSolidFuel(Scale biomassToSolidFuel) {
		this.biomassToSolidFuel = biomassToSolidFuel;
	}

	public Scale getBiomassToElectricityPotential() {
		return biomassToElectricityPotential;
	}
	

	public boolean isBiomassToElectricityPotential() {
		return biomassToElectricityPotential !=null;
	}

	public void setBiomassToElectricityPotential(Scale biomassToElectricityPotential) {
		this.biomassToElectricityPotential = biomassToElectricityPotential;
	}

	public Scale getBiomassToThermalPotential() {
		return biomassToThermalPotential;
	}
	

	public boolean isBiomassToThermalPotential() {
		return biomassToThermalPotential != null;
	}

	public void setBiomassToThermalPotential(Scale biomassToThermalPotential) {
		this.biomassToThermalPotential = biomassToThermalPotential;
	}

	@Override
	public Object copy(CopyBuilder copyBuilder) {
		return copyTo(new EnergySurvey(),copyBuilder);
	}

	@Override
	public void accept(FeatureVisitor visitor) {
		visitor.visit((ADEModelObject)this);
	}

	@Override
	public <T> T accept(FeatureFunctor<T> visitor) {
		return visitor.apply((ADEModelObject)this);
	}

	@Override
	public void accept(GMLVisitor visitor) {
		visitor.visit((ADEModelObject)this);
		
	}

	@Override
	public <T> T accept(GMLFunctor<T> visitor) {
		return visitor.apply((ADEModelObject)this);
	}	
	
}
