/*-
 * Copyright 2021 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.ade.foodwaterenergycitygml4j.landuse.model;



import org.citygml4j.builder.copy.CopyBuilder;
import org.citygml4j.model.citygml.ade.binding.ADEModelObject;
import org.citygml4j.model.common.visitor.FeatureFunctor;
import org.citygml4j.model.common.visitor.FeatureVisitor;
import org.citygml4j.model.common.visitor.GMLFunctor;
import org.citygml4j.model.common.visitor.GMLVisitor;
import org.citygml4j.model.gml.basicTypes.Code;
import org.citygml4j.model.gml.feature.AbstractFeature;
import org.citygml4j.model.gml.measures.Scale;


public class FoodSurvey extends AbstractFeature implements ADEModelObject{
		
	
	private static final long serialVersionUID = 6757445678768677685L;
	
	private Code foodCategory;
	private Scale foodDemand;
	private Scale foodProduction;
	private Scale foodProductionWaterDemand;
	private Scale foodProductionEnergyDemand;
	private Scale foodConsumption;
	
	
	
	
	public Code getFoodCategory() {
		return foodCategory;
	}
	public boolean isFoodCategory() {
		return foodCategory != null;
	}
	public void setFoodCategory(Code foodCategory) {
		this.foodCategory = foodCategory;
	}
	
	public Scale getFoodDemand() {
		return foodDemand;
	}
	

	public boolean isFoodDemand() {
		return foodDemand!=null;
	}
	public void setFoodDemand(Scale foodDemand) {
		this.foodDemand = foodDemand;
	}
	public Scale getFoodProduction() {
		return foodProduction;
	}
	public boolean isFoodProduction() {
		return foodProduction != null;
	}
	public void setFoodProduction(Scale foodProduction) {
		this.foodProduction = foodProduction;
	}
	public Scale getFoodProductionWaterDemand() {
		return foodProductionWaterDemand;
	}
	public boolean isFoodProductionWaterDemand() {
		return foodProductionWaterDemand!=null;
	}
	public void setFoodProductionWaterDemand(Scale foodProductionWaterDemand) {
		this.foodProductionWaterDemand = foodProductionWaterDemand;
	}
	public Scale getFoodProductionEnergyDemand() {
		return foodProductionEnergyDemand;
	}
	public boolean isFoodProductionEnergyDemand() {
		return foodProductionEnergyDemand!=null;
	}
	public void setFoodProductionEnergyDemand(Scale foodProductionEnergyDemand) {
		this.foodProductionEnergyDemand = foodProductionEnergyDemand;
	}
	public Scale getFoodConsumption() {
		return foodConsumption;
	}
	public boolean isFoodConsumption() {
		return foodConsumption!=null;
	}
	public void setFoodConsumption(Scale foodConsumption) {
		this.foodConsumption = foodConsumption;
	}
	@Override
	public Object copy(CopyBuilder copyBuilder) {
		return copyTo(new FoodSurvey(),copyBuilder);
	}

	@Override
	public void accept(FeatureVisitor visitor) {
		visitor.visit((ADEModelObject)this);
	}

	@Override
	public <T> T accept(FeatureFunctor<T> visitor) {
		return visitor.apply((ADEModelObject)this);
	}

	@Override
	public void accept(GMLVisitor visitor) {
		visitor.visit((ADEModelObject)this);
		
	}

	@Override
	public <T> T accept(GMLFunctor<T> visitor) {
		return visitor.apply((ADEModelObject)this);
	}
}
