/*-
 * Copyright 2021 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.ade.foodwaterenergycitygml4j.landuse.model;


import org.citygml4j.builder.copy.CopyBuilder;
import org.citygml4j.model.citygml.ade.binding.ADEModelObject;
import org.citygml4j.model.common.visitor.FeatureFunctor;
import org.citygml4j.model.common.visitor.FeatureVisitor;
import org.citygml4j.model.common.visitor.GMLFunctor;
import org.citygml4j.model.common.visitor.GMLVisitor;
import org.citygml4j.model.gml.feature.AbstractFeature;
import org.citygml4j.model.gml.measures.Area;


public class LandUseSurvey extends AbstractFeature implements ADEModelObject  {

	private static final long serialVersionUID = -3539870873714199259L;
	private Area buildingFootprintArea;
	private Integer population;
	private Integer surveyYear;
	private Double surveyYearVersion;
	private String surveyDescription;
	private Area surfaceArea;
	private FoodSurvey foodsurvey;
	private WaterSurvey watersurvey;
	private EnergySurvey energySurvey;

	
	public Area getBuildingFootprintArea() {
		return buildingFootprintArea;
	}
	
	public boolean isBuildingFootprintArea() {
		return buildingFootprintArea != null;
	}
	public void setBuildingFootprintArea(Area buildingFootprintArea) {
		this.buildingFootprintArea = buildingFootprintArea;
	}
	public Integer getPopulation() {
		return population;
	}
	
	public boolean isPopulation() {
		return population!=null;
	}
	public void setPopulation(Integer population) {
		this.population = population;
	}
	public Integer getSurveyYear() {
		return surveyYear;
	}
	public boolean isSurveyYear() {
		return surveyYear!= null;
	}
	public void setSurveyYear(Integer surveyYear) {
		this.surveyYear = surveyYear;
	}
	public Double getSurveyYearVersion() {
		return surveyYearVersion;
	}
	public boolean isSurveyYearVersion() {
		return surveyYearVersion != null;
	}
	public void setSurveyYearVersion(Double surveyYearVersion) {
		this.surveyYearVersion = surveyYearVersion;
	}
	public String getSurveyDescription() {
		return surveyDescription;
	}
	public boolean isSurveyDescription() {
		return surveyDescription != null;
	}
	public void setSurveyDescription(String surveyDescription) {
		this.surveyDescription = surveyDescription;
	}
	public Area getSurfaceArea() {
		return surfaceArea;
	}
	public boolean isSurfaceArea() {
		return surfaceArea != null;
	}
	public void setSurfaceArea(Area surfaceArea) {
		this.surfaceArea = surfaceArea;
	}
	

	public FoodSurvey getFoodsurvey() {
		return foodsurvey;
	}
	
	public boolean isFoodsurvey() {
		return foodsurvey!=null;
	}

	public void setFoodsurvey(FoodSurvey foodsurvey) {
		this.foodsurvey = foodsurvey;
	}
	
	

	public WaterSurvey getWatersurvey() {
		return watersurvey;
	}
	
	public boolean isWatersurvey() {
		return watersurvey!=null;
	}

	public void setWatersurvey(WaterSurvey watersurvey) {
		this.watersurvey = watersurvey;
	}

	public EnergySurvey getEnergySurvey() {
		return energySurvey;
	}
	
	public boolean isEnergySurvey() {
		return energySurvey != null;
	}

	public void setEnergySurvey(EnergySurvey energySurvey) {
		this.energySurvey = energySurvey;
	}

	@Override
	public Object copy(CopyBuilder copyBuilder) {
		return copyTo(new LandUseSurvey(),copyBuilder);
	}

	@Override
	public void accept(FeatureVisitor visitor) {
		visitor.visit((ADEModelObject)this);
	}

	@Override
	public <T> T accept(FeatureFunctor<T> visitor) {
		return visitor.apply((ADEModelObject)this);
	}

	@Override
	public void accept(GMLVisitor visitor) {
		visitor.visit((ADEModelObject)this);
		
	}

	@Override
	public <T> T accept(GMLFunctor<T> visitor) {
		return visitor.apply((ADEModelObject)this);
	}
	
	
	
	
	/*
	private List<FoodSurveyType> foodSurvey;
	private List<WaterSurveyType> waterSurvey;
	private List<EnergySurveyType> energySurvey;
	*/
	
	
	
}
